extern crate winapi;

use std::ffi::CString;
use winapi::um::fileapi::GetVolumeInformationA;
use encoding_rs::GBK;

extern crate log;

pub fn get_disk_info(drive: &str) -> Option<(String, String, u32)> {
    let mut volume_name: [i8; 100] = [0; 100];
    let mut file_system_name: [i8; 100] = [0; 100];
    let mut volume_serial_number: u32 = 0;
    let mut max_component_length: u32 = 0;
    let mut file_system_flags: u32 = 0;
    let tstr = CString::new(drive).unwrap();
    let success = unsafe {
        GetVolumeInformationA(
            tstr.as_ptr(),
            volume_name.as_mut_ptr(),
            volume_name.len() as u32,
            &mut volume_serial_number,
            &mut max_component_length,
            &mut file_system_flags,
            file_system_name.as_mut_ptr(),
            file_system_name.len() as u32,
        )
    };
    fn gbk2uft8(gbk: &[i8]) -> String {
        let unsigned_slice = gbk.iter().map(|&x| x as u8).collect::<Vec<u8>>();
        let (utf8, _, _) = GBK.decode(&unsigned_slice);
        utf8.replace("\0", "").to_string()
    }

    if success != 0 {
        let volume_label = gbk2uft8(&volume_name);
        let file_system_name = gbk2uft8(&file_system_name);
        log::info!("Volume Label: {}, file_system: {}, serial_number:{}\n", volume_label, file_system_name, volume_serial_number);
        return Some((volume_label, file_system_name, volume_serial_number));
    } else {
        log::debug!("volume {} Failed to get volume information.\n", drive);
        return None;
    }
}
#[test]
fn test_get_disk_info() {
    env_logger::init();
    let (label, file_system, serial_number) = get_disk_info("C:\\").unwrap();
    assert_eq!(label, "系统");
    assert_eq!(file_system, "NTFS");
    assert_ne!(serial_number, 0);

    let (label, file_system, serial_number) = get_disk_info("V:\\").unwrap();
    assert_eq!(label, "homes");
    assert_eq!(file_system, "NTFS");
    assert_ne!(serial_number, 0);
}
